import{V as n,D as u,b as m,a as l}from"./vRc2BJuY.js";import{A as p}from"./Dw7RWyOH.js";import{d as i,u as d,x as c,B as t,af as f,C as e,y as x,D as g}from"#entry";import"./DlAUqK2U.js";import"./D2lCyhx0.js";import"./BaCn2Olm.js";import"./DPQgZEES.js";import"./DIRTgmGz.js";import"./BPg7fJxw.js";import"./EcQIca6j.js";import"./CiZyL2M5.js";const k=i({__name:"new",setup(_){const a=d(),o=[{text:"AGENT GROUPS",disabled:!1,to:"/agent-groups/list",exact:!0},{text:"New",disabled:!0,to:"/agent-groups/new",exact:!0}];function s(){a.push({path:"/agent-groups/list"})}return(b,r)=>(x(),c("div",null,[t(f(u),{breadcrumbs:o,class:"mb-5"}),t(n,{variant:"outlined",class:"my-4"},{default:e(()=>[t(m,null,{default:e(()=>[...r[0]||(r[0]=[g(" Add a new Agent group ",-1)])]),_:1}),t(l,null,{default:e(()=>[t(p,{onCloseForm:s})]),_:1})]),_:1})]))}});export{k as default};
