import{d as n,T as i,y as c,C as o,z as l,A as r,B as t,D as s,Z as d,aG as p,_ as u}from"#entry";import{_ as m}from"./DlAUqK2U.js";import{V as f}from"./D2lCyhx0.js";const g=n({name:"DfScanProgress",props:{progress:{type:String,default:void 0},messageStatus:{type:String,default:void 0}},computed:{iconColor(){if(this.progress)switch(this.progress.toLowerCase()){case"done":return{icon:"mdi-check",color:"success",text:"Done"};case"in_progress":return{icon:"mdi-autorenew",color:"info",text:"Running"};case"not_started":return{icon:"mdi-timeline-plus-outline",color:"orange-darken-2",text:"Queued"};case"stopped":return{icon:"mdi-stop-circle",color:"red",text:"Stopped"};case"error":return{icon:"mdi-exclamation-thick",color:"purple",text:"Error"};case"disabled":return{icon:"mdi-stop-circle-outline",color:"grey",text:"Stopped"}}return{icon:"mdi-exclamation",color:"warning",text:""}}}});function x(e,S,_,h,C,V){return c(),i(f,{bottom:"",disabled:!e.messageStatus},{activator:o(({props:a})=>[t(u,p({color:e.iconColor.color,size:"small",label:"",variant:"flat"},a),{default:o(()=>[t(d,{size:"small",start:"",color:"white",class:"mb-n1"},{default:o(()=>[s(r(e.iconColor.icon),1)]),_:1}),s(" "+r(e.iconColor.text),1)]),_:1},16,["color"])]),default:o(()=>[l("span",null,r(e.messageStatus),1)]),_:1},8,["disabled"])}const k=m(g,[["render",x]]);export{k as D};
