import{J as b,K as h,bu as f,au as p,ao as k,ap as x,L as y,av as S,k as B,aP as w,aK as C,B as r,N as L,aA as V,aB as P}from"#entry";const T={actions:"button@2",article:"heading, paragraph",avatar:"avatar",button:"button",card:"image, heading","card-avatar":"image, list-item-avatar",chip:"chip","date-picker":"list-item, heading, divider, date-picker-options, date-picker-days, actions","date-picker-options":"text, avatar@2","date-picker-days":"avatar@28",divider:"divider",heading:"heading",image:"image","list-item":"text","list-item-avatar":"avatar, text","list-item-two-line":"sentences","list-item-avatar-two-line":"avatar, sentences","list-item-three-line":"paragraph","list-item-avatar-three-line":"avatar, paragraph",ossein:"ossein",paragraph:"text@3",sentences:"text@2",subtitle:"text",table:"table-heading, table-thead, table-tbody, table-tfoot","table-heading":"chip, text","table-thead":"heading@6","table-tbody":"table-row-divider@6","table-row-divider":"table-row, divider","table-row":"text@6","table-tfoot":"text@2, avatar@2",text:"text"};function A(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[];return r("div",{class:["v-skeleton-loader__bone",`v-skeleton-loader__${e}`]},[t])}function l(e){const[t,a]=e.split("@");return Array.from({length:a}).map(()=>i(t))}function i(e){let t=[];if(!e)return t;const a=T[e];if(e!==a){if(e.includes(","))return o(e);if(e.includes("@"))return l(e);a.includes(",")?t=o(a):a.includes("@")?t=l(a):a&&t.push(i(a))}return[A(e,t)]}function o(e){return e.replace(/\s/g,"").split(",").map(i)}const _=h({boilerplate:Boolean,color:String,loading:Boolean,loadingText:{type:String,default:"$vuetify.loading"},type:{type:[String,Array],default:"ossein"},...P(),...V(),...L()},"VSkeletonLoader"),E=b()({name:"VSkeletonLoader",props:_(),setup(e,t){let{slots:a}=t;const{backgroundColorClasses:s,backgroundColorStyles:d}=f(p(e,"color")),{dimensionStyles:u}=k(e),{elevationClasses:c}=x(e),{themeClasses:g}=y(e),{t:v}=S(),m=B(()=>i(w(e.type).join(",")));return C(()=>{const n=!a.default||e.loading;return r("div",{class:["v-skeleton-loader",{"v-skeleton-loader--boilerplate":e.boilerplate},g.value,s.value,c.value],style:[d.value,n?u.value:{}],"aria-busy":e.boilerplate?void 0:n,"aria-live":e.boilerplate?void 0:"polite","aria-label":e.boilerplate?void 0:v(e.loadingText),role:e.boilerplate?void 0:"alert"},[n?m.value:a.default?.()])}),{}}});export{E as V};
