"""create api key

Revision ID: cff0aef2c0f7
Revises: 4d13d49d324c
Create Date: 2024-05-29 17:08:31.735724

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "cff0aef2c0f7"
down_revision = "4d13d49d324c"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "api_key",
        sa.Column("key", sa.String(length=150), nullable=False),
        sa.PrimaryKeyConstraint("key"),
        sa.UniqueConstraint("key"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("api_key")
    # ### end Alembic commands ###
