import{b1 as e,x as o,y as r,z as t}from"#entry";import{_ as n}from"./DlAUqK2U.js";const a=e("/ripples_loading.svg"),c={},l={style:{position:"absolute",top:"50%",left:"50%",transform:"translate(-50%, -50%)"}};function i(p,s){return r(),o("div",l,[...s[0]||(s[0]=[t("img",{class:"d-block",style:{height:"200px"},src:a},null,-1),t("p",{class:"text-center"}," Loading... ",-1)])])}const m=n(c,[["render",i]]);export{m as S};
