"""Add references field to vulnerability

Revision ID: 7d12204c3f27
Revises: 746c5eb3d181
Create Date: 2022-10-25 22:47:46.208220

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "7d12204c3f27"
down_revision = "746c5eb3d181"
branch_labels = None
depends_on = None


def upgrade() -> None:
    op.add_column("vulnerability", sa.Column("references", sa.Text))


def downgrade() -> None:
    pass
