"""remove asset

Revision ID: cbb2e16e146a
Revises: 69c7d2f49a4e
Create Date: 2024-07-02 17:21:53.859101

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "cbb2e16e146a"
down_revision = "69c7d2f49a4e"
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("scan", schema=None) as batch_op:
        batch_op.drop_column("asset")
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("scan", schema=None) as batch_op:
        batch_op.add_column(sa.Column("asset", sa.VARCHAR(length=255), nullable=True))
    # ### end Alembic commands ###
