import{u as o}from"./DIRTgmGz.js";import{S as a}from"./kh6oSBUy.js";import{d as r,o as t,q as s,x as c,y as d,B as i,z as l}from"#entry";import{_ as m}from"./DlAUqK2U.js";const p=r({__name:"index",setup(_){const n=o().scanners;return t(()=>{n.length===0?s("/scanners"):s("/scan/list")}),(u,e)=>(d(),c("div",null,[i(a),e[0]||(e[0]=l("div",{class:"oxo-loader-bar"},null,-1))]))}}),b=m(p,[["__scopeId","data-v-f6b6d8b3"]]);export{b as default};
