import{J as p,K as v,k as S,h as V,O as w,Q as b,cA as A,bp as L}from"#entry";const o=["start","end","center"],g=["space-between","space-around","space-evenly"];function i(t,s){return A.reduce((n,e)=>{const a=t+L(e);return n[a]=s(),n},{})}const N=[...o,"baseline","stretch"],d=t=>N.includes(t),f=i("align",()=>({type:String,default:null,validator:d})),P=[...o,...g],y=t=>P.includes(t),C=i("justify",()=>({type:String,default:null,validator:y})),E=[...o,...g,"stretch"],j=t=>E.includes(t),m=i("alignContent",()=>({type:String,default:null,validator:j})),u={align:Object.keys(f),justify:Object.keys(C),alignContent:Object.keys(m)},h={align:"align",justify:"justify",alignContent:"align-content"};function G(t,s,n){let e=h[t];if(n!=null){if(s){const a=s.replace(t,"");e+=`-${a}`}return e+=`-${n}`,e.toLowerCase()}}const O=v({dense:Boolean,noGutters:Boolean,align:{type:String,default:null,validator:d},...f,justify:{type:String,default:null,validator:y},...C,alignContent:{type:String,default:null,validator:j},...m,...b(),...w()},"VRow"),T=p()({name:"VRow",props:O(),setup(t,s){let{slots:n}=s;const e=S(()=>{const a=[];let l;for(l in u)u[l].forEach(c=>{const k=t[c],r=G(l,c,k);r&&a.push(r)});return a.push({"v-row--no-gutters":t.noGutters,"v-row--dense":t.dense,[`align-${t.align}`]:t.align,[`justify-${t.justify}`]:t.justify,[`align-content-${t.alignContent}`]:t.alignContent}),a});return()=>V(t.tag,{class:["v-row",e.value,t.class],style:t.style},n.default?.())}});export{T as V};
