import{d as t,a0 as s,aO as a,b9 as n,T as i,g as p,y as m}from"#entry";import{u as c,e as u}from"./vRc2BJuY.js";import{u as l}from"./DIRTgmGz.js";import{V as d}from"./B0MvoCbX.js";import{_ as f}from"./DlAUqK2U.js";import"./D2lCyhx0.js";import"./Debz9GBs.js";import"./DicsgTSm.js";import"./D4UZsoDo.js";import"./BOdr3NOm.js";import"./EcQIca6j.js";const _=t({name:"Vulnerability",components:{VulnerabilityDetail:d},data(){return{scanner:{endpoint:"",name:"",apiKey:""}}},computed:{...a(l,["scanners"])},mounted(){const e=this.scanners.find(r=>u(r.endpoint)===(this._.provides[n]||this.$route).params.scanner);e===void 0?(this.reportError("Scanner not found"),this.$router.push({name:"scan-list"})):this.scanner=e},methods:{...s(c,["reportError"])}});function h(e,r,v,$,y,b){const o=p("VulnerabilityDetail");return m(),i(o,{"scan-id":(e._.provides[n]||e.$route).params.scan,scanner:e.scanner,"vuln-title":(e._.provides[n]||e.$route).params.vuln},null,8,["scan-id","scanner","vuln-title"])}const O=f(_,[["render",h]]);export{O as default};
