import{J as y,K as S,k,h as C,O as N,Q as h,cA as a,bp as f}from"#entry";const i=a.reduce((e,s)=>(e[s]={type:[Boolean,String,Number],default:!1},e),{}),d=a.reduce((e,s)=>{const t="offset"+f(s);return e[t]={type:[String,Number],default:null},e},{}),m=a.reduce((e,s)=>{const t="order"+f(s);return e[t]={type:[String,Number],default:null},e},{}),u={col:Object.keys(i),offset:Object.keys(d),order:Object.keys(m)};function L(e,s,t){let o=e;if(!(t==null||t===!1)){if(s){const r=s.replace(e,"");o+=`-${r}`}return e==="col"&&(o="v-"+o),e==="col"&&(t===""||t===!0)||(o+=`-${t}`),o.toLowerCase()}}const P=["auto","start","end","center","baseline","stretch"],V=S({cols:{type:[Boolean,String,Number],default:!1},...i,offset:{type:[String,Number],default:null},...d,order:{type:[String,Number],default:null},...m,alignSelf:{type:String,default:null,validator:e=>P.includes(e)},...h(),...N()},"VCol"),O=y()({name:"VCol",props:V(),setup(e,s){let{slots:t}=s;const o=k(()=>{const r=[];let n;for(n in u)u[n].forEach(l=>{const g=e[l],c=L(n,l,g);c&&r.push(c)});const b=r.some(l=>l.startsWith("v-col-"));return r.push({"v-col":!b||!e.cols,[`v-col-${e.cols}`]:e.cols,[`offset-${e.offset}`]:e.offset,[`order-${e.order}`]:e.order,[`align-self-${e.alignSelf}`]:e.alignSelf}),r});return()=>C(e.tag,{class:[o.value,e.class],style:e.style},t.default?.())}});export{O as V};
