# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/fingerprint/file/ios/metadata/metadata.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nLostorlab/agent/message/proto/v3/fingerprint/file/ios/metadata/metadata.proto\x12=ostorlab.agent.message.proto.v3.fingerprint.file.ios.metadata\"5\n\nPermission\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x19\n\x11usage_description\x18\x02 \x01(\t\"|\n\x07Message\x12\x11\n\tbundle_id\x18\x01 \x01(\t\x12^\n\x0bpermissions\x18\x02 \x03(\x0b\x32I.ostorlab.agent.message.proto.v3.fingerprint.file.ios.metadata.Permission')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.v3.fingerprint.file.ios.metadata.metadata_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _PERMISSION._serialized_start=143
  _PERMISSION._serialized_end=196
  _MESSAGE._serialized_start=198
  _MESSAGE._serialized_end=322
# @@protoc_insertion_point(module_scope)
