import{J as y,K as S,al as h,bK as O,b2 as x,k as e,r as T,aK as k,aV as p,b4 as K,bN as w,aG as c,bM as u,B as A}from"#entry";const B=S({id:String,text:String,...K(w({closeOnBack:!1,location:"end",locationStrategy:"connected",eager:!0,minWidth:0,offset:10,openOnClick:!1,openOnHover:!0,origin:"auto",scrim:!1,scrollStrategy:"reposition",transition:!1}),["absolute","persistent"])},"VTooltip"),G=y()({name:"VTooltip",props:B(),emits:{"update:modelValue":t=>!0},setup(t,d){let{slots:n}=d;const i=h(t,"modelValue"),{scopeId:v}=O(),g=x(),r=e(()=>t.id||`v-tooltip-${g}`),l=T(),f=e(()=>t.location.split(" ").length>1?t.location:t.location+" center"),m=e(()=>t.origin==="auto"||t.origin==="overlap"||t.origin.split(" ").length>1||t.location.split(" ").length>1?t.origin:t.origin+" center"),V=e(()=>t.transition?t.transition:i.value?"scale-transition":"fade-transition"),b=e(()=>c({"aria-describedby":r.value},t.activatorProps));return k(()=>{const P=u.filterProps(t);return A(u,c({ref:l,class:["v-tooltip",t.class],style:t.style,id:r.value},P,{modelValue:i.value,"onUpdate:modelValue":a=>i.value=a,transition:V.value,absolute:!0,location:f.value,origin:m.value,persistent:!0,role:"tooltip",activatorProps:b.value,_disableGlobalStack:!0},v),{activator:n.activator,default:function(){for(var a=arguments.length,s=new Array(a),o=0;o<a;o++)s[o]=arguments[o];return n.default?.(...s)??t.text}})}),p({},l)}});export{G as V};
